package test;


import java.io.File;
import java.util.Iterator;
import java.util.Properties;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.teneo.hibernate.HbDataStore;
import org.eclipse.emf.teneo.hibernate.HbHelper;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Environment;
import org.hibernate.dialect.HSQLDialect;

import com.example.testbag.SomeClass;
import com.example.testbag.SomeOtherClass;
import com.example.testbag.TestbagFactory;
import com.example.testbag.TestbagPackage;

public class TestBag extends TestCase {
    private static final String DBNAME="TestBagDB";
    
    private static Properties getHibernateProperties(String dbName) {
        final Properties props = new Properties(); 
        props.setProperty(Environment.DRIVER, "org.hsqldb.jdbcDriver");
        props.setProperty(Environment.USER, "sa");
        props.setProperty(Environment.URL, "jdbc:hsqldb:file:" + dbName + ";shutdown=true");
        props.setProperty(Environment.PASS, "");
        props.setProperty(Environment.DIALECT, HSQLDialect.class.getName());
        props.setProperty(Environment.CURRENT_SESSION_CONTEXT_CLASS, "thread");
        props.setProperty(Environment.STATEMENT_BATCH_SIZE, "1");
        return props;
    }

    public static void removeDBFiles(String dbname) {
        for (String dbFile: new File(".").list()) {
            File f = new File(dbFile);
            if (f.getName().startsWith(dbname)) {
                if (!f.delete()) {
                    throw new RuntimeException("Lock still held on DB file" + f.getName());
                }
            }
        }
    }

    /* (non-Javadoc)
     * @see junit.framework.TestCase#setUp()
     */
    @Override
    protected void setUp() throws Exception {
        removeDBFiles(DBNAME);
    }

    /* (non-Javadoc)
     * @see junit.framework.TestCase#tearDown()
     */
    @Override
    protected void tearDown() throws Exception {
        removeDBFiles(DBNAME);
    }

    private HbDataStore createHbDataStore(String dbname) {
        Properties props = getHibernateProperties(dbname);
        EPackage[] packages = { TestbagPackage.eINSTANCE };
        HbDataStore hbds = HbHelper.INSTANCE.createRegisterDataStore(dbname);
        hbds.setEPackages(packages);
        // set the relational database to use and other properties
        hbds.setHibernateProperties(props);
        hbds.setPersistenceProperties(props);
        hbds.initialize();
        return hbds;
    }
    
    public void test() {
        SessionFactory sessionFactory = createHbDataStore(DBNAME).getSessionFactory();
        
        try {
            Session session = sessionFactory.openSession();
            session.beginTransaction();

            SomeClass someClass = TestbagFactory.eINSTANCE.createSomeClass();
            session.save(someClass);

            SomeOtherClass otherClass = TestbagFactory.eINSTANCE.createSomeOtherClass();
            otherClass.setSomeProperty("foo");
            someClass.getRefs().add(otherClass);

            otherClass = TestbagFactory.eINSTANCE.createSomeOtherClass();
            otherClass.setSomeProperty("bar");
            someClass.getRefs().add(otherClass);

            session.getTransaction().commit();
            session.close();
        } finally {
            sessionFactory.close();
        }
        
        sessionFactory = createHbDataStore(DBNAME).getSessionFactory();
        try {
            Session session = sessionFactory.openSession();
            session.beginTransaction();

            Query query = session.createQuery("FROM SomeClass");          
            SomeClass someClass = (SomeClass)query.uniqueResult();
            Assert.assertNotNull(someClass);
            Assert.assertEquals(2, someClass.getRefs().size());
            Iterator i = someClass.getRefs().iterator();
            boolean found0 = false;
            boolean found1 = false;
            while (i.hasNext())
            {
                SomeOtherClass otherClass = (SomeOtherClass)i.next();
                if (otherClass.getSomeProperty().equals("foo")) {
                    found0 = true;
                } else if (otherClass.getSomeProperty().equals("bar")) {
                    found1 = true;
                }
            }
            if (!found0 || !found1) {
                Assert.fail("Missing some refs.");
            }
            session.close();
        } finally {
            sessionFactory.close();
        }
    }
}
